//
//  RCDTextMessageCell.m
//  RongIMKit_Demo
//
//  Created by Jue on 2019/8/12.
//  Copyright © 2019 RC. All rights reserved.
//

#import "RCDTextMessageCell.h"

#define RCDScreenWidth [UIScreen mainScreen].bounds.size.width
#define Font_Size 16
#define Extra_BackgroupView_CornerRadius 6.f

@interface RCDTextMessageCell ()

//翻译内容的 Label
@property (strong, nonatomic) UILabel *extraLabel;

//翻译内容的背景图
@property (strong, nonatomic) UIView *extraBackgroundView;

@end

@implementation RCDTextMessageCell

+ (CGSize)sizeForMessageModel:(RCMessageModel *)model
      withCollectionViewWidth:(CGFloat)collectionViewWidth
         referenceExtraHeight:(CGFloat)extraHeight {
    //翻译好的内容
    NSString *extra = model.extra;
    
    CGSize superSize = [super sizeForMessageModel:model withCollectionViewWidth:collectionViewWidth referenceExtraHeight:extraHeight];
    if (extra.length > 0) {
        CGSize extraSize = [RCDTextMessageCell getTextLabelSize:extra];
        CGFloat finalHeight = superSize.height + extraSize.height;
        return CGSizeMake(superSize.width, finalHeight);
    }else {
        return superSize;
    }
}

//计算文本 size 的方法
+ (CGSize)getTextLabelSize:(NSString *)content {
    if ([content length] > 0) {
        float maxWidth = RCDScreenWidth -
        (10 + [RCIM sharedRCIM].globalMessagePortraitSize.width + 10) * 2 - 5 - 35;
        CGRect textRect = [content
                           boundingRectWithSize:CGSizeMake(maxWidth, 8000)
                           options:(NSStringDrawingTruncatesLastVisibleLine | NSStringDrawingUsesLineFragmentOrigin |
                                    NSStringDrawingUsesFontLeading)
                           attributes:@{NSFontAttributeName : [UIFont systemFontOfSize:Font_Size]}
                           context:nil];
        textRect.size.height = ceilf(textRect.size.height);
        textRect.size.width = ceilf(textRect.size.width);
        return CGSizeMake(textRect.size.width + 5, textRect.size.height + 5);
    } else {
        return CGSizeZero;
    }
}

- (void)setDataModel:(RCMessageModel *)model {
    [super setDataModel:model];
    NSString *extra = model.extra;
    
    //为了防止复用问题的处理
    [self.extraLabel removeFromSuperview];
    [self.extraBackgroundView removeFromSuperview];
    
    //如果有翻译的内容，添加 UI，进行展示
    if (extra.length > 0) {
        CGSize extraSize = [[self class] getTextLabelSize:extra];
        CGRect superFrame = self.bubbleBackgroundView.frame;
        CGRect extraBackgroundViewFrame = CGRectZero;
        
        //判断消息方向，设置翻译背景图的 frame
        if (model.messageDirection == MessageDirection_SEND) {
            extraBackgroundViewFrame = CGRectMake(superFrame.origin.x + superFrame.size.width - extraSize.width - 15, superFrame.size.height + 3, extraSize.width + 10, extraSize.height + 6);
        } else {
            extraBackgroundViewFrame = CGRectMake(superFrame.origin.x + 5, superFrame.origin.y + superFrame.size.height + 3, extraSize.width + 10, extraSize.height + 6);
        }
        
        self.extraBackgroundView = [[UIView alloc] initWithFrame:extraBackgroundViewFrame];
        self.extraBackgroundView.backgroundColor = [UIColor whiteColor];
        self.extraBackgroundView.layer.cornerRadius = Extra_BackgroupView_CornerRadius;
        [self.messageContentView addSubview:self.extraBackgroundView];
        
        CGRect extraLabelFrame = CGRectMake(8, 3, extraSize.width, extraSize.height);
        self.extraLabel = [[UILabel alloc] initWithFrame:extraLabelFrame];
        self.extraLabel.font = [UIFont systemFontOfSize:Font_Size];
        self.extraLabel.numberOfLines = 0;
        [self.extraLabel setLineBreakMode:NSLineBreakByWordWrapping];
        [self.extraLabel setTextAlignment:NSTextAlignmentLeft];
        [self.extraBackgroundView addSubview:self.extraLabel];
        self.extraLabel.text = extra;
    }
}

@end
